/*
 * Copyright 2016-2020 Max Kellermann <max.kellermann@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "util/MimeType.hxx"

#include <gtest/gtest.h>

TEST(MimeType, Base)
{
	EXPECT_EQ("", GetMimeTypeBase(""));
	EXPECT_EQ("", GetMimeTypeBase(";"));
	EXPECT_EQ("foo", GetMimeTypeBase("foo"));
	EXPECT_EQ("foo/bar", GetMimeTypeBase("foo/bar"));
	EXPECT_EQ("foo/bar", GetMimeTypeBase("foo/bar;"));
	EXPECT_EQ("foo/bar", GetMimeTypeBase("foo/bar; x=y"));
	EXPECT_EQ("foo/bar", GetMimeTypeBase("foo/bar;x=y"));
}

TEST(UriUtil, Parameters)
{
	EXPECT_TRUE(ParseMimeTypeParameters("").empty());
	EXPECT_TRUE(ParseMimeTypeParameters("foo/bar").empty());
	EXPECT_TRUE(ParseMimeTypeParameters("foo/bar;").empty());
	EXPECT_TRUE(ParseMimeTypeParameters("foo/bar;garbage").empty());
	EXPECT_TRUE(ParseMimeTypeParameters("foo/bar; garbage").empty());

	auto p = ParseMimeTypeParameters("foo/bar;a=b");
	EXPECT_FALSE(p.empty());
	EXPECT_EQ(p["a"], "b");
	EXPECT_EQ(p.size(), 1u);

	p = ParseMimeTypeParameters("foo/bar; a=b;c;d=e ; f=g ");
	EXPECT_FALSE(p.empty());
	EXPECT_EQ(p["a"], "b");
	EXPECT_EQ(p["d"], "e");
	EXPECT_EQ(p["f"], "g");
	EXPECT_EQ(p.size(), 3u);
}
