/*
 * Copyright 2008-2019 Max Kellermann <max.kellermann@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * FOUNDATION OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CURL_INIT_HXX
#define CURL_INIT_HXX

#include "thread/Mutex.hxx"

class EventLoop;
class CurlGlobal;

/**
 * This class performs one-time initialization of libCURL and creates
 * one #CurlGlobal instance, shared across all #CurlInit instances.
 */
class CurlInit {
	static Mutex mutex;
	static unsigned ref;
	static CurlGlobal *instance;

public:
	explicit CurlInit(EventLoop &event_loop);
	~CurlInit() noexcept;

	CurlInit(const CurlInit &) = delete;
	CurlInit &operator=(const CurlInit &) = delete;

	CurlGlobal &operator*() noexcept {
		return *instance;
	}

	const CurlGlobal &operator*() const noexcept {
		return *instance;
	}

	CurlGlobal *operator->() noexcept {
		return instance;
	}

	const CurlGlobal *operator->() const noexcept {
		return instance;
	}
};

#endif
