/*
 * Copyright 2003-2021 The Music Player Daemon Project
 * http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include "FlacMetadataChain.hxx"
#include "FlacMetadataIterator.hxx"
#include "FlacIOHandle.hxx"
#include "FlacStreamMetadata.hxx"

bool
FlacMetadataChain::Read(InputStream &is) noexcept
{
	return Read(::ToFlacIOHandle(is), ::GetFlacIOCallbacks(is));
}

bool
FlacMetadataChain::ReadOgg(InputStream &is) noexcept
{
	return ReadOgg(::ToFlacIOHandle(is), ::GetFlacIOCallbacks(is));
}

void
FlacMetadataChain::Scan(TagHandler &handler) noexcept
{
	FlacMetadataIterator iterator(chain);

	do {
		FLAC__StreamMetadata *block = iterator.GetBlock();
		if (block == nullptr)
			break;

		flac_scan_metadata(block, handler);
	} while (iterator.Next());
}
