/*
 * Copyright 2003-2021 The Music Player Daemon Project
 * http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef MPD_ANDROID_CONTEXT_HXX
#define MPD_ANDROID_CONTEXT_HXX

#include "java/Object.hxx"

class AllocatedPath;
class AudioManager;

class Context : public Java::GlobalObject {
public:
	/**
	 * Global initialisation.  Looks up the methods of the
	 * Context Java class.
	 */
	static void Initialise(JNIEnv *env) noexcept;

	Context(JNIEnv *env, jobject obj) noexcept
		:Java::GlobalObject(env, obj) {}

	/**
	 * @param type the subdirectory name; may be nullptr
	 */
	[[gnu::pure]]
	AllocatedPath GetExternalFilesDir(JNIEnv *env,
					  const char *type=nullptr) noexcept;

	[[gnu::pure]]
	AllocatedPath GetCacheDir(JNIEnv *env) const noexcept;

	[[gnu::pure]]
	AudioManager *GetAudioManager(JNIEnv *env) noexcept;
};

#endif
